﻿/* ---------------------------------------------------------------------------------
Titre : Mesure de la fréquence de rotation en tr/mn d'un moteur POLOLU POL2280
Documentation : https://www.pololu.com/product/2280
Détails dans le fichier Lisez-moi.txt
-----------------------------------------------------------------------------------*/
using System;
using System.Threading;
using Microsoft.SPOT;
using Microsoft.SPOT.Hardware;
using SecretLabs.NETMF.Hardware;
using SecretLabs.NETMF.Hardware.NetduinoPlus;

using Microtoolskit.Hardware.Displays; // Contient la bibliothèque ELCD162

namespace NPM2280V1310
{   
    public class Program
    {
        // counter, counter_1 mémorisent le nombre de périodes délivrées par le codeur pendant un temps t = sample
        static UInt32 counter = 0, counter_1 = 0;

        enum Modes
        {
            ReglageVitesse,
            MesureTir,
            Transfert
        }
        static Modes mode = Modes.ReglageVitesse;

        public static void Main()
        {   // -----------------------------------------------------------------------------------------------------
            // Constantes
            // -----------------------------------------------------------------------------------------------------
            const UInt32 RazCount = 0;
            // -----------------------------------------------------------------------------------------------------
            // Variables
            // -----------------------------------------------------------------------------------------------------
            // Paramètres du timer
            var delay = 0;          // Attente avant échantillonnage de la fréquence de rotation du moteur
            // Paramètres des signaux PWM
            var F_PWM = 1000;       // Fréquence à appliquer au signal PWM
            var duty = 0;           // Rapport cyclique à appliquer au signal PWM;
            // Paramètres de l'application
            var sample = 0.018;     // Période d'échantillonnage de la fréquence de rotation du moteur (en seconde), 
                                    // à régler en fonctionde l'erreur acceptée sur la mesure (simulée dans PololuPOL2280_EssaisTir.xls)         
            var k = 1.25;           // n = k*F (voir Lisez-moi.txt) et de la fréquence de rotation du moteur         
            var nbMesure_n = 100;   // nombre de mesures pendant le tir
            // -----------------------------------------------------------------------------------------------------
            var n_tir = 0.0;        // Fréquence de rotation du moteur mesurée pendant le tir
            float[] table_n = new float[nbMesure_n]; // Tableau des mesures de la fréquence de rotation du moteur pendant le tir
            var i = 0;              // Compteur d'usage général         
            // -----------------------------------------------------------------------------------------------------
            // Réglage de la consigne de vitesse
            var RegleVitMot = new AnalogInput(Cpu.AnalogChannel.ANALOG_3);  // Consigne de vitesse réglée avec un potentiomètre sur In3

            // Commande d'un moteur POLOLU POL2280 sur la voie B de la carte Ardumoto
            PWM MoteurB = new PWM(PWMChannels.PWM_PIN_D11, F_PWM, duty, false);
            var SensB = new OutputPort(Pins.GPIO_PIN_D13, false);

            // Commande d'un moteur POLOLU POL2280 sur la voie A de la carte Ardumoto (sens inverse de B)
            PWM MoteurA = new PWM(PWMChannels.PWM_PIN_D3, F_PWM, duty, false);
            var SensA = new OutputPort(Pins.GPIO_PIN_D12, true);

            // LCD à connecter sur la sortie SERIE de la carte Tinkerkit pour libérer O5 (libre pour PWMA)           
            // Documentation de la classe ELCD162 : https://github.com/WebGE?tab=repositories
            // Création et initialisation d'un objet afficheur série ELCD-162 (par défaut: COM2, 19200, None, 8, 1)
            var Lcd = new ELCD162("COM1"); Lcd.Init(); Lcd.ClearScreen();

            // Acquisition du signal A du codeur sur l'entrée I0 de la carte Tinkerkit et du signal B sur l'entrée I1
            // On compte chaque front des deux signaux pour avoir N = 48 dans la relation N*Omega = 2*Pi*F
            // Avec N : nombre de périodes délivrées par l'encodeur pour un tour moteur
            //      F : fréquence du signal délivré par l'encodeur. Omega: vitesse angulaire du moteur en rad/s 
            var encodeurA = new InterruptPort(Pins.GPIO_PIN_A0, true, Port.ResistorMode.Disabled, Port.InterruptMode.InterruptEdgeBoth);
            var encodeurB = new InterruptPort(Pins.GPIO_PIN_A1, true, Port.ResistorMode.Disabled, Port.InterruptMode.InterruptEdgeBoth);

            // Le bouton-poussoir BTN3 du clavier Digilent déclenche la mesure pendant le tir (voir Graphe_Etats_Lanceur.docx)
            var BTN3 = new InputPort(Pins.GPIO_PIN_A2, false, Port.ResistorMode.Disabled);

            // Inscription des interruptions au gestionnaire d'évènement
            encodeurA.OnInterrupt += new NativeEventHandler(measureFreq);
            encodeurB.OnInterrupt += new NativeEventHandler(measureFreq);

            // Timer utilisé pour la remise à zéro du compteur après chaque échantillonnage de la fréquence de rotation du moteur
            encodeurA.DisableInterrupt(); encodeurB.DisableInterrupt();
            var timer = new Timer(new TimerCallback(SaveAndRazCounter), RazCount, delay, (int)(1000 * sample));
            MoteurB.Start(); MoteurA.Start(); 
            RegleVitMot.Scale = 0.75; // (0 a 6300tr/mn)
            encodeurA.EnableInterrupt(); encodeurB.EnableInterrupt();

            while (true)
            {
                if (BTN3.Read())
                    mode = Modes.MesureTir; 
                
                switch (mode)
                {       // Réglage et affichage de la consigne de  vitesse
                    case Modes.ReglageVitesse:
                        // La mesure de la fréquence de rotation du moteur est désactivée pendant l'affichage du réglage 
                        // de la consigne sur le lcd (pour les tests)
                        encodeurA.DisableInterrupt(); // Désactivation de l'interruption pour ne pas perturber le traitement et l'affichage   
                        encodeurB.DisableInterrupt();

                        // Mesure
                        var alpha = RegleVitMot.Read();          // mesure de la consigne de vitesse

                        // Traitement  
                        var Consigne_n = 9270 * alpha - 488;  // Calcul de la valeur de la consigne à afficher (voir PololuPOL2280_P.xlsx)
                        var n = k * (counter_1 / sample);     // Calcul de la fréquence de rotation n du moteur (pour N = 48)

                        // Affichage
                        Lcd.ClearScreen();
                        if (Consigne_n < 500)
                            Lcd.PutString("Csg= trop faible");
                        else
                            Lcd.PutString("Csg= " + Consigne_n.ToString("F0") + "tr/mn");
                            
                        Debug.Print("n = " + n.ToString("F1"));
                        Lcd.SetCursor(0, 1); Lcd.PutString("n  = " + n.ToString("F0") + "tr/mn"); // Affichage de la mesure de n
                        
                        MoteurB.DutyCycle = alpha; MoteurA.DutyCycle = alpha;
                        encodeurA.EnableInterrupt(); encodeurB.EnableInterrupt(); // Autorisation de l'interruption
                        Thread.Sleep(1000);
                    break;

                    // Accumulation dans un tableau, des valeurs de n mesurées pendant le tir 
                    case Modes.MesureTir: encodeurA.DisableInterrupt(); Lcd.ClearScreen(); Lcd.PutString("Mesure tir...");
                        encodeurA.EnableInterrupt(); encodeurB.EnableInterrupt();
                        i = 0;
                        do
                        {                     
                                n_tir = k * (counter_1 / sample);
                                table_n[i] = (float)n_tir;
                                i++; Thread.Sleep((int)(1000 * sample));
                        } while (i < table_n.Length);

                        mode = Modes.Transfert;
                    break;

                    case Modes.Transfert: encodeurA.DisableInterrupt(); Lcd.ClearScreen(); Lcd.PutString("Transfert");
                       
                        for (int j = 0; j < table_n.Length; j++)
                        {
                            Debug.Print(j + " : " + table_n[j].ToString("F0"));
                        }
                    
                        Thread.Sleep(2000);
                        mode = Modes.ReglageVitesse;
                    break;

                    default:
                        break;
                }
                
            }
        }

        // ---------------------------------------------------------------------------------------------
        // Gestionnaires d'interruption
        // ---------------------------------------------------------------------------------------------
        // Raz compteur
        static void SaveAndRazCounter(Object RazCount)
        {
            counter_1 = counter; // Mémorisation de la valeur mesurée
            counter = (UInt32)RazCount;
        }
        // ---------------------------------------------------------------------------------------------
        // Incrémentation du compteur
        static void measureFreq(uint port, uint state, DateTime time)
        {
            counter++;
        }
        // ---------------------------------------------------------------------------------------------
    }
}
